(*A globally declared variable can take one of the four types of expressions as its values*)
type datatype =
	Int|
        String|
        Boolean|
        AVP
(*An expression in NwQL could be literals of one of the four data types, an identifier and an assignment expression*)
type expr =
        IntLit of int |
        StrLit of string |
        BoolLit of bool |
        AVPList of  (string * string)  list|
        Id of string |
        LitAssign of string * expr
(*global variable declaration is a record of three items i.e. one of the four datatypes, the name of the variable and its value *)
type gvdecl = 
{
	vtype:datatype;
	vname:string;
	value:expr;
}
(*Following types of statements are possible in NWQL, a statement constitute an independently compilable program in NWQL*)
type stmt=
	Expr of expr
	|
	Select of string list * string (*select bar from tablename*)
	|
	Insert of string * string list * expr list  (*insert into foo (column1,column2,column3) (value1,value2,value3) *)
	|
	Delete of expr list* expr * string
(*A program is zero of more global variable declarations as well as zero or more statements *) 
type program =
	gvdecl list * stmt list
(************************************************************************
Name=ptydatatype
Description=Print string of the data type
Input= One of the four possible data types for a global variable
Output= String of the data type of global variable
*************************************************************************)
let ptydatatype=function
	Int -> "int"
	|
	String -> "string"
	|
	Boolean -> "boolean"
	|
	AVP -> "avp"
(************************************************************************
Name=ptyexpr
Description=Print string of expression types
Input= One of the supported types of expressions 
Output= String of the expression types
*************************************************************************)
let rec ptyexpr = function
(*matching pattern for integer type of literals*)
	IntLit(lit) -> string_of_int lit 
	|
(*matching pattern for string literals*)
	StrLit(lit) -> lit 
	|
(*matching pattern for bollean literals*)
	BoolLit(lit) -> string_of_bool lit 
	|
(*matching pattern for identifiers*)
	Id(lit) -> lit
	|
(*matching patterns for the literals of type AVPLit*)
	AVPList(listofavps) -> 
		let rec ptyavp = function 
		[]-> "here"
		|
		hd::tail -> (fst hd)  ^  (snd hd) ^ (ptyavp tail)
	in ptyavp listofavps
	|
(*matching pattern for the expression of type assignment*)
	LitAssign(name,value) -> name ^ " " ^ (ptyexpr value)
(************************************************************************
Name=ptygvdecl
Description=Print string of gobal variable declarations
Input= variable declaration
Output= String of the variable declaration
*************************************************************************)
let ptygvdecl  globalvar =
	(ptydatatype globalvar.vtype) ^ (globalvar.vname) ^ (ptyexpr globalvar.value)
(************************************************************************
Name=ptystmt
Description=Print string of one of the types of statements
Input= one of the statement types
Output= String of the statements
*************************************************************************)
let ptystmt = function
	Expr(expr) -> ptyexpr expr
	|
(*select*)
	Select(columnslist,tablename) -> 
		"select " ^ String.concat "," columnslist ^ " from " ^ tablename ^ "\n"
	|
	Insert(table,columnnamelist,valueslist) ->
	"insert into "^table^" "^ (String.concat "," columnnamelist)^" VALUES "^(String.concat "," (List.map ptyexpr valueslist))
	|
	Delete(flownames,nodeid,tablename) ->
			"delete " ^ (String.concat "," (List.map ptyexpr flownames)) ^ "in " ^ (ptyexpr nodeid) ^  " from " ^tablename^ "\n"
(************************************************************************
Name=ptyprogram
Description= Print string of a program
Input= A nwql program
Output= String of a NWQL program
*************************************************************************)
let ptyprogram (idvarlist,stmtlist) =
	String.concat "" (List.map ptygvdecl idvarlist) ^
	String.concat "" (List.map ptystmt stmtlist)
