open Past
(*Code generation module*)

(*Generating code for datatype*)
(*There is no datatype in python so no code will be generated*)
let string_of_pdatatype = function
	P_int -> ""
	|
        P_string -> ""
	|
        P_boolean -> ""
	|
        P_avp-> ""

(************************************************************************
Name=removequotes
Description=This method is used to remove redundant quotes 
in string and AVP type of expressions
Input=An expression
Output=An expression with redundant quotes removed if the expression
ins of type string or AVP
*************************************************************************)
let removequotes pexpr =
match pexpr with P_strLit(str) ->
	if (String.get str 0) = '"' then
		P_strLit(String.sub str 1 ((String.length str) -2))
	else 
		P_strLit(str)
|
P_avpList(avplist) -> 
	let removeavpquotes=function(key,value) -> 
	if (String.get key 0) = '"'
		then 
			(String.sub key 1 ((String.length key) -2),String.sub value 1 ((String.length value) -2))
		else
			(key,value)
	in
	let newlist = List.map removeavpquotes avplist
	in
	P_avpList(newlist)
|
_ as other -> other

(************************************************************************
Name=trans_link link
Description=This method is useful for tranforming 
the generic hyperlink for the specific hyperlink
required for the statement of type Insert
Input=A string containing hyperlink
Outpput=A string containing hyperlink required for Insert statement
*************************************************************************)
let trans_link link =
	(*Find index of , since in a table ',' first apprears to separate authentication credentials from the hyperlink*)
	let index=String.index link ','
	(*length of the link*)
	and length=String.length link
	in
	(*Return a tuple whose first member is hyperlink and whose second member is authentication info*)
	(String.sub link 0 index,String.sub link (index+1) (length-(index+1)))

(************************************************************************
Name=indexof
Description=This method is useful for getting index of a particular 
element in a list
Input=A list and an element
Output=Index of the element
*************************************************************************)
let rec indexof clist elem=
	match clist with [] -> raise(Failure("No columns mentioned")) 
	|
	hd::tail ->
		if hd=elem 
			then 0
		else
			1+ indexof tail elem

(************************************************************************
Name=string_of_pexpr
Description=This method is used to generate python code for expressions
Input= Expression
Output= String representing the respective python code
*************************************************************************)
let rec string_of_pexpr= function
	(*An integer literal in NwQL will simply be compiled in to integer literal in python *)
        P_intLit(lit) -> string_of_int lit
	|
	(*A string literal in NwQL will simply be complied in to string literal in python*)
        P_strLit(lit) ->  lit 
	|
	(*A boolean literal in NwQL will simply be compiled in to boolean literal in python with first letter of the literal capitalised*)
	P_boolLit(lit) -> String.capitalize(string_of_bool lit)
	|
	P_avpList(lit) ->
		let rec translate_AVPList = function 
		[] -> ""
		|
		hd::tail -> (fst hd) ^  "," ^  (snd hd) ^  "," ^  (translate_AVPList tail) in
			let dict =translate_AVPList lit in
			let endpos= String.rindex dict ','
			in "[" ^ (String.sub dict 0 endpos) ^ "]"
	|
	(*An identifier in NwQL is simply compiled in to a python identifier*)
	P_id(id) -> id
	|
	(*Literal assignment*)
	P_litAssign(id,expr) -> id ^ "=" ^ string_of_pexpr expr

(************************************************************************
Name=string_of_insert_pexpr
Description=This method is used to generate python code for expressions
used as AVPs in insert statements
Input=Expression
Output= String representing the respective python code
*************************************************************************)
let rec string_of_insert_pexpr= function
        P_intLit(lit) -> "\"" ^ (string_of_int lit) ^ "\""
        |
        (*A string literal in NwQL will simply be complied in to string literal in python*)
        P_strLit(lit) ->   "\"" ^ lit ^ "\"" 
        |
        (*A boolean literal in NwQL will simply be compiled in to boolean literal in python with first letter of the literal capitalised*)
        P_boolLit(lit) -> String.capitalize(string_of_bool lit)
        |
        (*An identifier in NwQL is simply compiled in to a python identifier*)
        P_id(id) -> id
        |
	_ -> raise(Failure("Unsupport expression in the statement"))

(************************************************************************
Name=string_of_action
Description=This method is used to generate python code for action column
used in insert statements
Input= Expression of type AVP
Output= String representing the respective python code
*************************************************************************)

let string_of_action =function P_avpList(lit) ->
	let rec string_of_avp = function [] -> ""
	|
	hd::tl -> 
		if (fst hd) = "DROP"
			then "[\"DROP\"],"
		else  "[\""^(fst hd) ^ "=" ^(snd hd) ^"\"],"

	in string_of_avp lit
	|
	_ as hello-> (string_of_insert_pexpr hello)

let string_of_node=function P_strLit(lit) ->
	"{\"id\":"^"\""^lit^"\",\"type\":\"OF\"},"
	|P_id(id)->"{\"id\":"^id^",\"type\":\"OF\"},"
	|_ ->""


let fold_func data cname expr =
	if cname="actions" then  data ^  "\"" ^ cname ^ "\"" ^  ":" ^(string_of_action expr)
	else
		if cname="node" then data ^  "\"" ^ cname ^ "\"" ^  ":" ^  (string_of_node expr)
		else
        	data ^  "\"" ^ cname ^ "\"" ^  ":" ^  (string_of_insert_pexpr expr)  ^ ","


let isflownameID =function P_id(_) -> true
|
_ -> false


let isnodenameID= function P_id(_) -> true
|
_ -> false
(************************************************************************
Name=string_of_pgvdecl
Description= Generating code for global variable declaration
Input= A product type of gvdecl
Output= A String representing global variable declaration in python
*************************************************************************)
let string_of_pgvdecl  globalvar=
	globalvar.pvname ^"=" ^string_of_pexpr globalvar.pvalue

(************************************************************************
Name=string_of_pstmt
Description=Generating code for statements
Input=A select statement
Output= A String representing statements in python
*************************************************************************)
let string_of_pstmt = function
	P_expr(pexpr) -> string_of_pexpr pexpr
	|
	Pselect(columnname,tablename) ->
		"try:\n"
			^
			"\t select=requests.get("^tablename^")\n"
			^
			"\t if (select.status_code == requests.codes.ok):\n"
				^
				(*Building a list of keys*)
				let rec col_string= function 
					[] -> ""
					|
					hd::tl -> "\"" ^ hd ^ "\"," ^ (col_string tl)
					in "\t\tcollist=[" ^ col_string columnname^"]\n"
					^
					"\t\tselect=select.json()\n"
					^
					"\t\tflow_entries=select[\'flowConfig\']\n"
					^
					"\t\tfor i in range(len(flow_entries)):\n"
					^
						"\t\t\tprint(\'\\n\\n\\n\')\n"
						^
						"\t\t\tprint(\'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\')\n"
						^
						"\t\t\tfor j in range(len(collist)):\n"
						^
							"\t\t\t\tif collist[j] in flow_entries[i]:\n"
							^
								"\t\t\t\t\tprint(\'-------------------------------------------------------------------------------\')\n"
							^
								"\t\t\t\t\tprint(collist[j],flow_entries[i][collist[j]])\n"
							^
						"\t\t\tprint(\'+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\')\n"
						^
						"\t\t\tprint(\'\\n\\n\\n\')\n"

			^
			"\t else:\n"
				^
				"\t\t print(\"HTTP error\",select.status_code)\n"

		^
		"except requests.ConnectionError:\n"
			^
			"\t print(\"Connection Unsuccessful\")\n"
	|
	Pinsert(tablename,cnamelist,exprlists) ->
		let exprlist=List.map removequotes exprlists
		in
		(*first we are going to get a hyperlink and authentication credentials separate*)
		let hylink,auth=trans_link tablename in
		(*now we get the length of the hyperlink*)
		let hylink_len=String.length hylink in
		(*now we removed the singles quotes from the hyperlink*)
		let hylink_sub=String.sub hylink 1 (hylink_len-2)
		in
		(*get the index of flowname from the columnnames tables*)
		let fnameind=indexof cnamelist "name"
		and
		(*get the index of nodename from the columnnames tables*)
		nodenameind=indexof cnamelist "node"
		in 
		(*got the value of the flowname*)
		let fname=List.nth exprlist fnameind
		and
		(*got the value of the nodename*)
		nodename=List.nth exprlist nodenameind
		in 
		(*fname_removed_*)
		let url=hylink_sub^"/node/OF/"^(ptypexpr nodename)^"/staticFlow/"^(ptypexpr fname)
		in
		let putdata="{ \"installInHw\":\"true\","  ^ (List.fold_left2 fold_func "" cnamelist exprlist) ^ "}"
		in
		(if ((isnodenameID nodename) && (isflownameID fname)) then "url="^"\""^hylink_sub^"/node/OF/\"+"^(ptypexpr nodename)^"+\"/staticFlow/\""^"+"^(ptypexpr	fname)^"\n"
		else if (isnodenameID nodename) then  "url="^"\""^hylink_sub^"/node/OF/\"+"^(ptypexpr nodename)^"+\"/staticFlow/"^(ptypexpr fname)^"\"\n"
		else if (isflownameID fname) then  "url="^"\""^hylink_sub^"/node/OF/"^(ptypexpr nodename)^"/staticFlow/"^"\""^"+"^(ptypexpr fname)^"\n"
		else
		"url=\""^url^"\"\n"
		)
		^"putdata="^putdata^"\n"^
		"headers={\"content-type\":\"application/json\"}\n"^
		"try:\n"^
			"\t insert=requests.put(url,"^auth^",data=json.dumps(putdata),headers=headers)\n"^
			"\t if (insert.status_code==requests.codes.ok):\n"^
				"\t\t print(\"Flow entry successfully updated\")\n"^
			"\t elif (insert.status_code==requests.codes.created):\n"^
				"\t\t print(\"Flow entry successfully created\")\n"^
			"\t else:\n"^
				"\t\t print(\"HTTP error\",insert.status_code)\n"^
		"except requests.ConnectionError:\n"^
		"\t print(\"Connection Unsuccessful\")\n"

	|
	Pdelete(flowlist,nodeid,tname) ->
	let compile_delete (flow,node,table) =
		let flowname=removequotes flow
		and
		nodename=removequotes node in
			let hylink,auth=trans_link table in
				let hylink_len=String.length hylink in
					let hylink_sub=String.sub hylink 1 (hylink_len-2) in 
						let url=hylink_sub^"/node/OF/"^(ptypexpr nodename)^"/staticFlow/"^(ptypexpr flowname) in
						(if ((isnodenameID nodename) && (isflownameID flowname)) 
							then "url="^"\""^hylink_sub^"/node/OF/\"+"^(ptypexpr nodename)^"+\"/staticFlow/\""^"+"^(ptypexpr  flowname)^"\n"
						else if (isnodenameID nodename) 
							then  "url="^"\""^hylink_sub^"/node/OF/\"+"^(ptypexpr nodename)^"+\"/staticFlow/"^(ptypexpr flowname)^"\"\n"
						else if (isflownameID flowname) 
							then  "url="^"\""^hylink_sub^"/node/OF/"^(ptypexpr nodename)^"/staticFlow/"^"\""^"+"^(ptypexpr flowname)^"\n"
						else
                					"url=\""^url^"\"\n"
        					)
						^"headers={\"content-type\":\"application/json\"}\n"^
						"try:\n"^
							"\t remove=requests.delete(url,"^auth^",headers=headers)\n"^
								"\t if (remove.status_code==204):\n"^
									"\t\t print(\"Flow entry successfully removed from the controller\")\n"^
								"\t else:\n"^
                                					"\t\t print(\"HTTP error\",remove.status_code)\n"^
                				"except requests.ConnectionError:\n"^
                					"\t print(\"Connection Unsuccessful\")\n"
	in
		let join elem1 elem2 elem3 = (elem1,elem2,elem3) in
                                        let par_join expr=join expr nodeid tname in
						let list_of_deletes= List.map par_join flowlist in
							let answer=List.map compile_delete list_of_deletes in
								String.concat "" answer

(************************************************************************
Name=string_of_progrm
Description=Generating code for a NWQL program
Input=A NWQL program
Output= An executable python program
*************************************************************************)
let string_of_program (pgvdecllist,pstmtlist)=
	"import requests \n" ^
	"import json\n"^
        String.concat "\n" (List.map string_of_pgvdecl pgvdecllist) ^ "\n" ^
        String.concat "\n" (List.map string_of_pstmt pstmtlist)
