%{ open Ast %}
/* beginning of declaration section */

%token ASTERISK,COLON,COMMA,LEFTPAREN,RIGHTPAREN,SEMICOLON,LEFTCURLY,RIGHTCURLY,ASSIGN,HASH
/*punctuation and separators*/ 

%token SELECT,INSERT,DELETE,INTO,FROM,VALUES,IN
/* keywords */

/*data types*/
%token INT,STRING,BOOLEAN,AVP

%token <int> INTLIT 
/*integer literal */

%token <string> STRLIT 
/*string literals */

%token <bool> BOOLLIT 
/* boolean literals */

%token <string> ID 
/* identifier */

%token <string> TNAME

%token <string*string>AVPLIT

%token EOF 
/*End of file symbol */

%start program 
/*start symbol is a program */

%type <Ast.program> program
%%
/*End of declaration section */

/*Beginning of the rules section */

vdecl:
datatype ID ASSIGN expr{{vtype=$1;vname=$2;value=$4}}

vdecllist:
HASH{[]}
|
vdecllist vdecl SEMICOLON {$2::$1}

datatype: 
INT{Int}
|
STRING{String}
|
BOOLEAN{Boolean}
|
AVP{AVP}

program:
vdecllist HASH statements {
	(
	if (List.length $1>0) && (List.length $3>0)
	then
		List.rev $1, List.rev $3
	else
		if(List.length $1<1) && (List.length $3>0)
		then
			[{vtype=String;vname="dummy";value=StrLit("\"dummy\"")}],List.rev $3

	
		else 
			raise(Failure("A program should have at least one statement"))
	)
	}

statements:
/* a single statement */
stmt {[$1]}
|
statements SEMICOLON stmt {($3::$1)}
/* or a list of statements where statements are separated by colons */

stmt:
expr{Expr($1)}
/*An statement is either an expression ,see expr */
|
SELECT columnnames FROM TNAME{Select($2,$4)} 
|
INSERT INTO TNAME LEFTPAREN columnnames RIGHTPAREN VALUES LEFTPAREN expressions RIGHTPAREN {Insert($3,$5,$9)}
|
DELETE expressions IN expr FROM TNAME{Delete($2,$4,$6)}
/* or a delete statement */

columnnames:
ID {[$1]}
|
ID COMMA columnnames {($1::$3)}
/*a list of columnname=value used in insert statement*/

avplist:
AVPLIT {[$1]}
|
AVPLIT COMMA avplist {($1::$3)}

expr:
LEFTCURLY avplist RIGHTCURLY {AVPList($2)}
|
INTLIT{IntLit($1)}
|
STRLIT{StrLit($1)}
|
BOOLLIT{BoolLit($1)}
|
ID ASSIGN expr{LitAssign($1,$3)}
|
ID{Id($1)}

/* expressions */

expressions:
expr{[$1]}
|
expr COMMA expressions {$1::$3}
