(*This file is a python abstract syntax tree*)
(*The goal of python compiler/translator will be to translate an ast generated in NwQL to Python*)

(* Similar to a NwQL program the translated python also begins with global variable declarations of zero or more variables*)
(*There are four data types in NwQL integers, strings, boolean and an aggregate data type called AVP and the shall be translated in to
int, string, boolean and tuples respectively in NwQL*)

type pdatatype =
        P_int|
        P_string|
        P_boolean|
        P_avp
(*An expression in NwQL could be literals of one of the four data types, an identifier and an assignment expression*)
type pexpr =
        P_intLit of int |
        P_strLit of string |
        P_boolLit of bool |
        P_avpList of  (string * string)  list|
        P_id of string |
        P_litAssign of string * pexpr

type pgvdecl =
{
        pvtype:pdatatype;
        pvname:string;
        pvalue:pexpr;
}

(*a NwQL program consist of a list of global declarations and a list of statements*)

type pstmt=
        P_expr of pexpr
	|
	Pselect of string list * string
	|
	Pinsert of string * string list * pexpr list
	|
	Pdelete of pexpr list * pexpr * string

(*A program is zero of more global variable declarations as well as zero or more statements *)
type pprogram =
        pgvdecl list * pstmt list

let rec ptypexpr = function
(*matching pattern for integer type of literals*)
        P_intLit(lit) -> string_of_int lit
        |
(*matching pattern for string literals*)
        P_strLit(lit) -> lit
        |
(*matching pattern for bollean literals*)
        P_boolLit(lit) -> string_of_bool lit
        |
(*matching pattern for identifiers*)
        P_id(lit) -> lit
        |
(*matching patterns for the literals of type AVPLit*)
        P_avpList(listofavps) ->
                let rec ptyavp = function
                []-> ""
                |
                hd::tail -> (fst hd)^(snd hd)^ (ptyavp tail)
        in ptyavp listofavps
        |
(*matching pattern for the expression of type assignment*)
        P_litAssign(name,value) -> name ^ (ptypexpr value)
