{open Parser
}
(*declaring some common regular expressions*)

(*regular expression for table name in select, insert and delete statements*)
let tname=(''''h''t''t''p')[^'\n']*

(*identifier is used in multiple regular expression*)
let identifier =['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']+

(*string literal is used in multiple regular expressions *)
let strlit ='"'(([^ '"'])*)'"'

(*this regular expression matches any of the whitespace characters and the associated semantic action calls the token rule back. *)
rule token = parse
[' ' '\t' '\r' '\n'] {token lexbuf}
|
"/*" {comment lexbuf} 
|
"select"{SELECT} | "from"{FROM} | "insert"{INSERT} | "into"{INTO} | "values"{VALUES} |"delete"{DELETE}|"in"{IN}
|
'#'{HASH}
|
"int" {INT} | "string" {STRING} | "boolean"{BOOLEAN}|"avp"{AVP}
|
['0'-'9']+ as intlit {INTLIT(int_of_string intlit)}
|
"true"|"false" as boollit {BOOLLIT(bool_of_string boollit)}
|
tname as tablename {TNAME(tablename)}
|
identifier  as ident {ID(ident)}
|
strlit as stringliteral {STRLIT(stringliteral)}
|
'('(strlit as id)':'(strlit as lit)')'{AVPLIT(id,lit)}
(*Regular expression punctuation and separators*)
|'*'{ASTERISK}|','{COMMA}|'('{LEFTPAREN}|')'{RIGHTPAREN}|';'{SEMICOLON} |'{'{LEFTCURLY}|'}'{RIGHTCURLY}|'='{ASSIGN} 
|
eof {EOF}
(*EOF*)
|
_ as char {raise (Failure("Illegal character " ^ Char.escaped char))} 

and 
comment= parse
"*/" {token lexbuf}
|
_ {comment lexbuf}
